@@ -27,6 +27,10 @@
 --
 --
 -- $Log: backup.c,v $
+-- Revision 1.21  2008/06/26 14:51:33  jsedwards
+-- Changed nwos_initialize_backup to return false if opening backup files
+-- failed.
+--
 -- Revision 1.20  2008/06/21 15:06:19  jsedwards
 -- Added code to log backup file name when opening.
 --
@@ -134,7 +138,7 @@ typedef struct {
 } Dummy_Block;
 
 
-void nwos_initialize_backup(const char* backup_location, Disk_Header* disk_header)
+bool nwos_initialize_backup(const char* backup_location, Disk_Header* disk_header)
 {
     char* username;
     TimeStamp time_stamp;
@@ -144,6 +148,7 @@ void nwos_initialize_backup(const char* backup_location, Disk_Header* disk_heade
     size_t bytes_written();
     int retry_count = 0;
     char buf[128];
+    bool result = true;
 
     assert(backup_location != NULL);
 
@@ -195,7 +200,9 @@ void nwos_initialize_backup(const char* backup_location, Disk_Header* disk_heade
 
 	if (backup_file_desc < 0)
 	{
+	    fprintf(stderr, "Error creating backup file ");
 	    perror(nwos_backup_path);
+	    result = false;
 	}
 	else
 	{
@@ -209,18 +216,22 @@ void nwos_initialize_backup(const char* backup_location, Disk_Header* disk_heade
 		perror(nwos_backup_path);
 		close(backup_file_desc);
 		backup_file_desc = -1;
+		result = false;
 	    }
 
 	    memset(buffer, 0, sizeof(buffer));
 
-	    if (write(backup_file_desc, buffer, sizeof(buffer)) != sizeof(buffer))
+	    if (result && write(backup_file_desc, buffer, sizeof(buffer)) != sizeof(buffer))
 	    {
 		perror(nwos_backup_path);
 		close(backup_file_desc);
 		backup_file_desc = -1;
+		result = false;
 	    }
 	}
     }
+
+    return result;
 }
 
 
