@@ -27,6 +27,26 @@
 --
 --
 -- $Log: disc_list.c,v $
+-- Revision 1.26  2008/07/26 15:09:02  jsedwards
+-- Changed to use returned value from nwos_read_files_disc_list for exit code
+-- if it fails.
+--
+-- Revision 1.25  2008/07/24 16:40:00  jsedwards
+-- Moved read_file_names function to file.c and rename to
+-- nwos_read_files_disc_list.
+--
+-- Revision 1.24  2008/07/24 14:20:59  jsedwards
+-- Change to read subdirectories recursively.
+--
+-- Revision 1.23  2008/07/24 11:25:18  jsedwards
+-- Move code that scans for all file names out of "main" and into new function
+-- "read_file_names".  Also moved "num_files", "file_names" and "files"
+-- variables out of "main" and made global to this file.
+--
+-- Revision 1.22  2008/05/22 10:04:55  jsedwards
+-- Fix Bug #1954154 - Changed so that revised file links are in the PATH AND
+-- FILE ASSOCIATION objects instead of the FILE objects.
+--
 -- Revision 1.21  2008/03/30 13:38:45  jsedwards
 -- Moved code to get disc ID before checking public files as well.
 --
@@ -119,16 +139,21 @@
 --
 */
 
-
+#include <assert.h>
 #include <dirent.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
 #include <time.h>
 #include "objectify_private.h"     /* this is kinda bad, but need overwrite to link old and new revisions */
 #include "crc32.h"
 
 
+static char* file_names[MAX_FILES_PER_DISC_LIST];
+static ObjRef files[MAX_FILES_PER_DISC_LIST];
+
+
 static size_t get_path_object_size(void* file_path_obj)
 {
     assert(((C_struct_File_Path*)file_path_obj)->count > 0);
@@ -139,10 +164,9 @@ static size_t get_path_object_size(void* file_path_obj)
 
 void check_for_previous_version_and_link(ObjRef* new_assoc_ref)
 {
+    C_struct_Path_And_File_Association assoc_obj;
     C_struct_Path_And_File_Association old_assoc_obj;
     C_struct_Path_And_File_Association new_assoc_obj;
-    C_struct_File old_file;
-    C_struct_File new_file;
     uint8 kludge[FILE_BLOCK_SIZE];
     C_struct_File_Path* ptr_path_obj = (C_struct_File_Path*)kludge;
     ReferenceList* ref_list;
@@ -152,9 +176,7 @@ void check_for_previous_version_and_link(ObjRef* new_assoc_ref)
 
     nwos_read_object_from_disk(new_assoc_ref, &new_assoc_obj, sizeof(new_assoc_obj));
 
-    nwos_read_object_from_disk(&new_assoc_obj.file, &new_file, sizeof(new_file));
-
-    assert(is_void_reference(&new_file.header.object.prev_version));
+    assert(is_void_reference(&new_assoc_obj.header.object.prev_version));
 
     nwos_read_variable_sized_object_from_disk(&new_assoc_obj.path, kludge, sizeof(kludge), &get_path_object_size);
 
@@ -173,31 +195,42 @@ void check_for_previous_version_and_link(ObjRef* new_assoc_ref)
 	    {
 		nwos_read_object_from_disk(&ref_list->references[i], &old_assoc_obj, sizeof(old_assoc_obj));
 
-		nwos_read_object_from_disk(&old_assoc_obj.file, &old_file, sizeof(old_file));
-
 		printf("Old association: %08x  file: %08x  next_version: %08x\n",
 		       nwos_ref_to_word(&ref_list->references[i]),
 		       nwos_ref_to_word(&old_assoc_obj.file),
-		       nwos_ref_to_word(&old_file.header.object.next_version));
+		       nwos_ref_to_word(&old_assoc_obj.header.object.next_version));
 
-		if (is_void_reference(&old_file.header.object.next_version)) break;
+		if (is_void_reference(&old_assoc_obj.header.object.next_version)) break;
 	    }
 	}
 
 	if (i < num_refs - 1)
 	{
-	    copy_reference(&old_file.header.object.next_version, &new_assoc_obj.file);
-	    copy_reference(&new_file.header.object.prev_version, &old_assoc_obj.file);
+	    assert(is_same_object(&new_assoc_obj.header.common.id, new_assoc_ref));
+	    assert(is_void_reference(&old_assoc_obj.header.object.next_version));
+	    assert(is_void_reference(&new_assoc_obj.header.object.prev_version));
+	    assert(is_same_object(&old_assoc_obj.path, &new_assoc_obj.path));
 
-	    nwos_crc32_calculate((uint8*) &old_file.header.object, sizeof(ObjectHeader), old_file.header.common.header_chksum);
-	    nwos_crc32_calculate((uint8*) &new_file.header.object, sizeof(ObjectHeader), new_file.header.common.header_chksum);
+	    copy_reference(&old_assoc_obj.header.object.next_version, &new_assoc_obj.header.common.id);
+	    copy_reference(&new_assoc_obj.header.object.prev_version, &old_assoc_obj.header.common.id);
+
+	    nwos_crc32_calculate((uint8*) &old_assoc_obj.header.object, sizeof(ObjectHeader), old_assoc_obj.header.common.header_chksum);
+	    nwos_crc32_calculate((uint8*) &new_assoc_obj.header.object, sizeof(ObjectHeader), new_assoc_obj.header.common.header_chksum);
+
+	    nwos_overwrite_object_to_disk(&old_assoc_obj.header.common.id, &old_assoc_obj, sizeof(old_assoc_obj));
+	    nwos_overwrite_object_to_disk(&new_assoc_obj.header.common.id, &new_assoc_obj, sizeof(new_assoc_obj));
 
-	    nwos_overwrite_object_to_disk(&old_assoc_obj.file, &old_file, sizeof(old_file));
-	    nwos_overwrite_object_to_disk(&new_assoc_obj.file, &new_file, sizeof(new_file));
 	    printf("linked - old: %08x and new: %08x\n",
-		   nwos_ref_to_word(&old_assoc_obj.file),
-		   nwos_ref_to_word(&new_assoc_obj.file));
-	  }
+		   nwos_ref_to_word(&old_assoc_obj.header.common.id),
+		   nwos_ref_to_word(&new_assoc_obj.header.common.id));
+
+	    nwos_read_object_from_disk(&old_assoc_obj.header.common.id, &assoc_obj, sizeof(assoc_obj));
+	    assert(memcmp(&assoc_obj, &old_assoc_obj, sizeof(assoc_obj)) == 0);
+
+	    nwos_read_object_from_disk(&new_assoc_obj.header.common.id, &assoc_obj, sizeof(assoc_obj));
+	    assert(memcmp(&assoc_obj, &new_assoc_obj, sizeof(assoc_obj)) == 0);
+	}
+
     }
 }
 
@@ -217,12 +250,8 @@ int main(int argc, char* argv[])
     char id[14];
     char *p;
     bool ok = false;
-    char* file_names[MAX_FILES_PER_DISC_LIST];
     int num_files;
-    ObjRef files[MAX_FILES_PER_DISC_LIST];
     ObjRef disc_list_ref;
-    DIR* dp;
-    struct dirent *dir_entry;
     bool add_revision = false;
     bool ignore_public = false;
     bool public_file_mismatch = false;
@@ -255,50 +284,24 @@ int main(int argc, char* argv[])
     /* First open the directory and read all of the file names */
     /***********************************************************/
 
-    dp = opendir(argv[argi]);
-    if (dp == NULL)
-    {
-	perror(argv[argi]);
-	exit(1);
-    }
+    num_files = nwos_read_files_disc_list(argv[argi], NULL, file_names, 0);    /* NULL for subdirectory indicates this is the root */
 
-    num_files = 0;
-    dir_entry = readdir(dp);
-    while (dir_entry != NULL)
+    if (num_files < 1)
     {
-	if (strcmp(dir_entry->d_name, ".") != 0 && strcmp(dir_entry->d_name, "..") != 0)
+	if (num_files == 0)
 	{
-	    if (num_files == MAX_FILES_PER_DISC_LIST)
-	    {
-		fprintf(stderr, "Too many files in directory: %d, currently can only handle %d files max\n",
-			num_files, MAX_FILES_PER_DISC_LIST);
-		exit(1);
-	    }
-
-	    file_names[num_files] = malloc(strlen(dir_entry->d_name) + 1);
-
-	    if (file_names[num_files] == NULL)
-	    {
-		perror("allocating memory for file name");
-		exit(1);
-	    }
-
-	    strcpy(file_names[num_files], dir_entry->d_name);
-
-	    /* printf("%s\n", file_names[num_files]); */
-
-	    num_files++;
+	    fprintf(stderr, "Error: directory '%s' is empty!\n", argv[argi]);
 	}
 
-	dir_entry = readdir(dp);
+	fprintf(stderr, "\n");
+
+	exit(-num_files);   /* exit code is error code returned by nwos_read_files_disc_list changed to positive */
     }
 
-    if (closedir(dp) != 0)
-    {
-	perror(argv[argi]);
-	exit(1);
-    }
 
+    /*********************************/
+    /* Get the pass phrase from user */
+    /*********************************/
 
     nwos_get_key_from_password(big_key, sizeof(big_key));
 
