@@ -27,6 +27,13 @@
 --
 --
 -- $Log: disk_io.c,v $
+-- Revision 1.87  2008/07/19 13:34:12  jsedwards
+-- Added new disk_io_is_read_only and disk_io_is_read_write functions.
+--
+-- Revision 1.86  2008/07/16 14:23:47  jsedwards
+-- Fixed copy and paste error, where chunk_info .used and .index were written
+-- into .ref when moving entry.
+--
 -- Revision 1.85  2008/06/26 14:54:30  jsedwards
 -- Change so that if initialize backup fails, it shuts disk_io down again and
 -- exits.
@@ -954,6 +961,20 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 }
 
 
+/****************/
+/* Open status. */
+/****************/
+
+bool nwos_disk_io_is_read_only()
+{
+    return read_only;
+}
+
+bool nwos_disk_io_is_read_write()
+{
+    return !read_only;
+}
+
 
 /*********************************************/
 /* Function to terminate the disk I/O module */
@@ -1102,8 +1123,8 @@ void nwos_allocate_new_chunk(uint32 ref)
 	if (chunk_info[i].ref < ref) break;
 
 	chunk_info[i].ref = chunk_info[i-1].ref;
-	chunk_info[i].ref = chunk_info[i-1].used;
-	chunk_info[i].ref = chunk_info[i-1].index;
+	chunk_info[i].used = chunk_info[i-1].used;
+	chunk_info[i].index = chunk_info[i-1].index;
     }
 
     offset = (ref - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK;
