@@ -27,6 +27,9 @@
 --
 --
 -- $Log: log_disc.c,v $
+-- Revision 1.35  2008/07/25 12:11:03  jsedwards
+-- Changed to allow it to scan subdirectories recursively.
+--
 -- Revision 1.34  2008/03/25 14:18:40  jsedwards
 -- Added an error message if the disk is empty so the user doesn't get some
 -- bogus assert.
@@ -154,6 +157,9 @@
 
 #define MAX_STORAGE_LOCATIONS 48  /* for now */
 
+static char* file_names[MAX_FILES_PER_DISC_LIST];
+static ObjRef files[MAX_FILES_PER_DISC_LIST];
+
 
 static void get_input(char* descr, char* buffer, size_t size)
 {
@@ -193,9 +199,7 @@ int main(int argc, char* argv[])
     char *p;
     bool ok = false;
     int binder = -2;
-    char* file_names[MAX_FILES_PER_DISC_LIST];
     int num_files;
-    ObjRef files[MAX_FILES_PER_DISC_LIST];
     ObjRef location_class_ref;
     ObjRef object_class;
     C_struct_Class_Definition class_def_obj;
@@ -214,8 +218,6 @@ int main(int argc, char* argv[])
 #endif
     uint8 kludge[MAX_SIZE_DISC_LIST];
     C_struct_Disc_List* ptr_list_obj = (C_struct_Disc_List*)kludge;
-    DIR* dp;
-    struct dirent *dir_entry;
 
 
     if (argc != 2)
@@ -224,54 +226,23 @@ int main(int argc, char* argv[])
 	exit(1);
     }
 
-    dp = opendir(argv[1]);
-    if (dp == NULL)
-    {
-	perror(argv[1]);
-	exit(1);
-    }
 
-    num_files = 0;
-    dir_entry = readdir(dp);
-    while (dir_entry != NULL)
+    /******************************************************/
+    /* First open the disc and read all of the file names */
+    /******************************************************/
+
+    num_files = nwos_read_files_disc_list(argv[1], NULL, file_names, 0);    /* NULL for subdirectory indicates this is the root */
+
+    if (num_files < 1)
     {
-	if (strcmp(dir_entry->d_name, ".") != 0 && strcmp(dir_entry->d_name, "..") != 0)
+	if (num_files == 0)
 	{
-	    if (num_files == MAX_FILES_PER_DISC_LIST)
-	    {
-		fprintf(stderr, "Too many files in directory: %d, currently can only handle %d files max\n",
-			num_files, MAX_FILES_PER_DISC_LIST);
-		exit(1);
-	    }
-
-	    file_names[num_files] = malloc(strlen(dir_entry->d_name) + 1);
-
-	    if (file_names[num_files] == NULL)
-	    {
-		perror("allocating memory for file name");
-		exit(1);
-	    }
-
-	    strcpy(file_names[num_files], dir_entry->d_name);
-
-	    /* printf("%s\n", file_names[num_files]); */
-
-	    num_files++;
+	    fprintf(stderr, "\nERROR: Disc is empty!  Perhaps it is not mounted?\n\n");
 	}
 
-	dir_entry = readdir(dp);
-    }
+	fprintf(stderr, "\n");
 
-    if (closedir(dp) != 0)
-    {
-	perror(argv[1]);
-	exit(1);
-    }
-
-    if (num_files == 0)
-    {
-	fprintf(stderr, "\nERROR: Disc is empty!  Perhaps it is not mounted?\n\n");
-	exit(1);
+	exit(1);   /* read_file_names found invalid file types */
     }
 
     printf("\n");
