@@ -27,6 +27,48 @@
 --
 --
 -- $Log: objectify.c,v $
+-- Revision 1.159  2008/07/19 14:40:55  jsedwards
+-- Added a hack to allow the root object to have a public class definition and
+-- write_object_to_disk to not assert.
+--
+-- Revision 1.158  2008/07/19 13:40:57  jsedwards
+-- Added code in get_file_class_reference to check to see if the latest file
+-- class revision has been added if it didn't exist when first called.
+--
+-- Revision 1.157  2008/07/17 13:12:03  jsedwards
+-- Fix nwos_write_object_to_disk and nwos_overwrite_object_to_disk to verify
+-- the class_definition of the object is of the correct type, public or
+-- private.
+--
+-- Revision 1.156  2008/07/16 02:32:24  jsedwards
+-- Fix Bug #2016420 - "After expanding a compressed 0026 file, bad checksum
+-- error".  The get_file_002_reference did not return the correct reference
+-- when there was no private file_001 reference.  Replaced code in the
+-- get_file_00X_reference functions with a call to the new get_file_references
+-- that finds all of the old file references correctly.
+--
+-- Revision 1.155  2008/07/15 19:10:41  jsedwards
+-- Added asserts in old get_00x_file_reference functions to verify that the
+-- one put in the table by get_file_references function matches.
+--
+-- Revision 1.154  2008/07/15 14:41:46  jsedwards
+-- Fixed get_file_references function to get private references now.
+--
+-- Revision 1.153  2008/07/15 14:08:11  jsedwards
+-- Added new get_file_references function that for now just reads the public
+-- file class definitions.
+--
+-- Revision 1.152  2008/05/24 11:01:49  jsedwards
+-- Changed file_XXX_referenc variables to file_references[] array.
+--
+-- Revision 1.151  2008/05/24 10:46:30  jsedwards
+-- Moved get_file_class_reference function after the get_file_XXX_reference
+-- functions.  NO code changes!
+--
+-- Revision 1.150  2008/05/24 10:44:06  jsedwards
+-- Moved 'result' variables out of the get_file_XXX_reference functions and
+-- made static variables (global to this file) for them.
+--
 -- Revision 1.149  2008/05/06 13:31:03  jsedwards
 -- Changed return type of nwos_read_object_from_disk_and_decrypt to bool
 -- instead of void and changed it to return false if the block read fails.
@@ -1087,6 +1129,8 @@ void nwos_initialize_objectify(uint8 bf_key[16], uint32 linear_seed, uint32 seri
 
 /* returns true if the object was created, false if it already existed */
 
+static bool creating_root = false;
+
 bool nwos_create_root(ObjRef* ref)
 {
     C_struct_Root root_obj;
@@ -1122,7 +1166,9 @@ bool nwos_create_root(ObjRef* ref)
 
 	nwos_crc32_calculate((uint8*) &root_obj.class_definition_class, sizeof(root_obj) - sizeof(EveryObject), root_obj.header.common.data_chksum);
 
+	creating_root = true;
 	nwos_write_object_to_disk(ref, &root_obj, sizeof(root_obj));
+	creating_root = false;
 
 #ifdef PUBLIC_MODE
 	nwos_add_to_references(ref, &root_class_ref);
@@ -1516,129 +1562,146 @@ bool nwos_read_object_from_disk_and_decrypt(ObjRef* ref, void* object, size_t si
 
 
 #ifndef USE_PREDEFINED_STRUCTS
-static ObjRef* get_file_class_reference()
-{
-    static ObjRef result;
 
-    if (is_void_reference(&result))
+/* These should only be referenced by using the get_file_XXX_reference functions below! */
+static ObjRef public_file_references[3];
+static ObjRef file_references[3];
+
+static void get_file_references()
+{
+    static volatile bool done = false;
+    ObjRef class_ref;
+    EveryObject header;
+    int i;
+
+    /* can't do anything until everything is initialized */
+    if (nwos_ready && !done)
     {
-        /* can't do anything until everything is initialized */
-	if (nwos_ready)
+	done = true;   /* in case we get called recursively */
+
+	/* we know that as of version 0027 there are 3 versions of the file class */
+	nwos_find_public_class_definition("FILE", &public_file_references[2]);
+
+	nwos_read_object_headers_from_disk(&public_file_references[2], &header);
+	assert(!is_void_reference(&header.object.prev_version));
+	assert(is_void_reference(&header.object.next_version));
+
+	copy_reference(&public_file_references[1], &header.object.prev_version);
+
+	nwos_read_object_headers_from_disk(&public_file_references[1], &header);
+	assert(!is_void_reference(&header.object.prev_version));
+	assert(is_same_object(&header.object.next_version, &public_file_references[2]));
+
+	copy_reference(&public_file_references[0], &header.object.prev_version);
+
+	nwos_read_object_headers_from_disk(&public_file_references[0], &header);
+	assert(is_void_reference(&header.object.prev_version));
+	assert(is_same_object(&header.object.next_version, &public_file_references[1]));
+
+	copy_reference(&file_references[0], &public_file_references[0]);
+	copy_reference(&file_references[1], &public_file_references[1]);
+	copy_reference(&file_references[2], &public_file_references[2]);
+
+#ifndef PUBLIC_MODE
+	if (nwos_find_private_class_definition("FILE", &class_ref))       /* are there any private files? */
 	{
-#ifdef PUBLIC_MODE
-	    nwos_find_public_class_definition("FILE", &result);
-#else
-	    /* in case there isn't a private one, make non-void so won't rerun this each time */
-	    if (!nwos_find_private_class_definition("FILE", &result))
+	    nwos_read_object_headers_from_disk(&class_ref, &header);
+
+	    assert(is_void_reference(&header.object.next_version));
+	    assert(!is_void_reference(&header.object.clone_of));
+
+	    for (i = 2; i >= 0; i--)
 	    {
-		result.word = 0xffffffff;
+		if (is_same_object(&public_file_references[i], &header.object.clone_of))
+		{
+		    copy_reference(&file_references[i], &class_ref);
+		    break;
+		}
+	    }
+
+	    while (i > 0)
+	    {
+		assert(!is_void_reference(&header.object.prev_version));
+
+		copy_reference(&class_ref, &header.object.prev_version);
+
+		if (nwos_reference_type(&class_ref) == Public_Reference) break;
+
+		nwos_read_object_headers_from_disk(&class_ref, &header);
+
+		assert(is_same_object(&header.object.next_version, &file_references[i]));
+
+		i--;
+
+		assert(is_same_object(&public_file_references[i], &header.object.clone_of));
+
+		copy_reference(&file_references[i], &class_ref);
 	    }
-#endif
 	}
+#endif
     }
 
-    return &result;
+#if 0
+    printf("public_file_reference[0]: %08x\n", nwos_ref_to_word(&public_file_references[0]));
+    printf("public_file_reference[1]: %08x\n", nwos_ref_to_word(&public_file_references[1]));
+    printf("public_file_reference[2]: %08x\n", nwos_ref_to_word(&public_file_references[2]));
+    printf("\n");
+    printf("file_reference[0]: %08x\n", nwos_ref_to_word(&file_references[0]));
+    printf("file_reference[1]: %08x\n", nwos_ref_to_word(&file_references[1]));
+    printf("file_reference[2]: %08x\n", nwos_ref_to_word(&file_references[2]));
+#endif
 }
 
 
 static ObjRef* get_file_001_reference()
 {
-    static ObjRef result;
-    static volatile bool inside_already = false;
-    ObjRef class_ref;
-    EveryObject header;
-
-    if (is_void_reference(&result))
+    if (is_void_reference(&file_references[0]))
     {
-        /* can't do anything until everything is initialized */
-	if (nwos_ready && !inside_already)
-	{
-	    inside_already = true;   /* in case we get called recursively */
-
-#ifdef PUBLIC_MODE
-	    nwos_find_public_class_definition("FILE", &class_ref);
-	    nwos_read_object_headers_from_disk(&class_ref, &header);
-	    assert(!is_void_reference(&header.object.prev_version));  /* there has to be at least one */
-#else
-	    /* in case there isn't an old one, make something non-void so won't rerun this each time */
-	    result.word = 0xffffffff;
-
-	    if (nwos_find_private_class_definition("FILE", &class_ref))
-	    {
-		nwos_read_object_headers_from_disk(&class_ref, &header);
-
-		if (is_void_reference(&header.object.prev_version))  /* updated class not created yet */
-		{
-		    copy_reference(&result, &class_ref);
-		}
-	    }
-	    else
-	    {
-		void_reference(&header.object.prev_version);
-	    }
-#endif
-	    while(!is_void_reference(&header.object.prev_version))
-	    {
-		copy_reference(&result, &header.object.prev_version);
-		nwos_read_object_headers_from_disk(&result, &header);
-	    }
-
-	    inside_already = false;
-	}
+	get_file_references();
     }
 
-    return &result;
+    return &file_references[0];
 }
 
+
 static ObjRef* get_file_002_reference()
 {
-    static ObjRef result;
-    static volatile bool inside_already = false;
-    ObjRef class_ref;
-    EveryObject header;
-
-    if (is_void_reference(&result))
+    if (is_void_reference(&file_references[1]))
     {
-        /* can't do anything until everything is initialized */
-	if (nwos_ready && !inside_already)
-	{
-	    inside_already = true;   /* in case we get called recursively */
-
-#ifdef PUBLIC_MODE
-	    nwos_find_public_class_definition("FILE", &class_ref);
-	    nwos_read_object_headers_from_disk(&class_ref, &header);
-	    assert(!is_void_reference(&header.object.prev_version));  /* there has to be at least one */
-#else
-	    /* in case there isn't an old one, make something non-void so won't rerun this each time */
-	    result.word = 0xffffffff;
-
-	    if (nwos_find_private_class_definition("FILE", &class_ref))
-	    {
-		nwos_read_object_headers_from_disk(&class_ref, &header);
-
-		if (is_void_reference(&header.object.prev_version))  /* updated class not created yet */
-		{
-		    copy_reference(&result, &class_ref);
-		}
-	    }
-	    else
-	    {
-		void_reference(&header.object.prev_version);
-	    }
-#endif
-	    while(!is_void_reference(&header.object.prev_version))
-	    {
-		copy_reference(&result, &header.object.prev_version);
-		nwos_read_object_headers_from_disk(&result, &header);
-	    }
-
-	    copy_reference(&result, &header.object.next_version);
-
-	    inside_already = false;
-	}
+	get_file_references();
     }
 
-    return &result;
+    return &file_references[1];
+}
+
+
+static ObjRef* get_file_class_reference()
+{
+    if (is_void_reference(&file_references[2]))
+    {
+	get_file_references();
+    }
+#ifndef PUBLIC_MODE
+    /* if storage is writable check to see if an updated file class has been created since this was first initialized */
+    else if (nwos_disk_io_is_read_write() && nwos_reference_type(&file_references[2]) == Public_Reference)
+    {
+	ObjRef class_ref;
+	EveryObject header;
+
+	if (nwos_find_private_class_definition("FILE", &class_ref))       /* see if one has been created since */
+	{
+	    nwos_read_object_headers_from_disk(&class_ref, &header);
+
+	    if (is_same_object(&header.object.clone_of, &public_file_references[2]))
+	    {
+		copy_reference(&file_references[2], &class_ref);
+		printf("Updating file class reference 2: %08x\n", nwos_ref_to_word(&file_references[2]));
+	    }
+	}
+    }
+#endif
+
+    return &file_references[2];
 }
 #endif
 
@@ -1953,13 +2016,15 @@ void nwos_write_object_to_disk(ObjRef* ref, void* object, size_t size)
     int i;
     int seq = 0;
 
-
     memset(ivec, 0, sizeof(ivec));
 
 
     assert(sizeof(extra) == FILE_BLOCK_SIZE);
 
-#ifndef PUBLIC_MODE
+#ifdef PUBLIC_MODE
+    assert(nwos_reference_type(&((EveryObject*)object)->common.class_definition) == Public_Reference);
+#else
+    assert(nwos_reference_type(&((EveryObject*)object)->common.class_definition) == Private_Reference || creating_root);
     assert(nwos_block_used(ref));      /* assert bit in map was set by generate id */
 #endif
 
@@ -2019,6 +2084,13 @@ void nwos_overwrite_object_to_disk(ObjRef* ref, void* object, size_t size)
 
     assert(nwos_object_exists(ref));
 
+#ifdef PUBLIC_MODE
+    assert(nwos_reference_type(&((EveryObject*)object)->common.class_definition) == Public_Reference);
+#else
+    assert(nwos_reference_type(&((EveryObject*)object)->common.class_definition) == Private_Reference);
+    assert(nwos_block_used(ref));      /* assert bit in map was set by generate id */
+#endif
+
     memset(ivec, 0, sizeof(ivec));
 
     nwos_write_object_to_disk_and_encrypt(ref, object, size, ivec, nwos_random_sequence[0]);
