@@ -27,6 +27,12 @@
 --
 --
 -- $Log: objectify.h,v $
+-- Revision 1.145  2008/07/24 16:43:16  jsedwards
+-- Added external declaration for nwos_read_files_disc_list.
+--
+-- Revision 1.144  2008/07/24 11:39:22  jsedwards
+-- Changed MAX_FILES_PER_DISC_LIST from 4096 to 1000000.
+--
 -- Revision 1.143  2008/07/19 11:26:23  jsedwards
 -- Added external declarations for progress bar functions.
 --
@@ -1090,10 +1096,11 @@ extern bool nwos_file_is_identical(ObjRef* assoc_ref, const char* path, MatchCod
 extern const char* nwos_check_invalid_type(mode_t mode);
 extern void nwos_check_directory_is_all_ok(const char* path);
 
-#define MAX_FILES_PER_DISC_LIST 4096
+#define MAX_FILES_PER_DISC_LIST 1000000
 #define MAX_SIZE_DISC_LIST (sizeof(C_struct_Disc_List) + sizeof(ObjRef) * MAX_FILES_PER_DISC_LIST)
 extern size_t nwos_get_disc_list_object_size(void* disc_list_obj);
 extern bool nwos_find_disc_list(char id[12], ObjRef* ref);
+extern int nwos_read_files_disc_list(const char* root_dir, const char* subdirectory, char* file_names[MAX_FILES_PER_DISC_LIST], int num_files);
 extern bool nwos_find_matching_disc_list(ObjRef files[MAX_FILES_PER_DISC_LIST], ObjRef* ref);
 extern ObjCreateResult nwos_create_disc_list(char id[12], ObjRef disc_list[MAX_FILES_PER_DISC_LIST], ObjRef* ref);
 extern bool nwos_find_disc_copy(ObjRef* disc_list, int copy_num, ObjRef* ref);
