@@ -1,6 +1,6 @@
 /*
 --             This file is part of the New World OS project
---                 Copyright (C) 2004-2007  QRW Software
+--                 Copyright (C) 2004-2008  QRW Software
 --           J. Scott Edwards - j.scott.edwards.nwos@gmail.com 
 --                      http://www.qrwsoftware.com
 --                      http://nwos.sourceforge.com
@@ -27,6 +27,23 @@
 --
 --
 -- $Log: reference_list.c,v $
+-- Revision 1.33  2008/07/19 13:59:15  jsedwards
+-- Changed nwos_add_to_references to call nwos_get_object_class_without_update
+-- instead of nwos_get_object_class.  This fixed a problem where cloning the
+-- updated file class definition called this function to add the new reference
+-- to the class definition reference list.  But when this called
+-- nwos_get_object_class, it called get_file_00x_reference, which called
+-- get_file_references, which called find_private_class_defintion, which reads
+-- the class defintion reference list.  Unfortunately that was what was being
+-- added with this function so it wasn't in the list yet so it could not find
+-- the "FILE" class definition, then later puked because it didn't have the
+-- private FILE class definitions in the file_references array.
+--
+-- Revision 1.32  2008/05/06 13:27:26  jsedwards
+-- Added assert around calls to nwos_read_object_from_disk_and_decrypt because
+-- it now returns true or false if it read the object okay.  Before it asserted
+-- itself if it could not read the block.
+--
 -- Revision 1.31  2007/09/16 15:00:48  jsedwards
 -- Fixed case where entry to be removed is in first block and there are two
 -- or more blocks in the list.
@@ -227,7 +244,7 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
     cache->first_block->next_block_ptr = NULL;
 
-    nwos_read_object_from_disk_and_decrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]);
+    assert(nwos_read_object_from_disk_and_decrypt(&cache->ref, &cache->first_block->list, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[0]));
 
     assert(is_same_object(&cache->first_block->list.common_header.class_definition, &nwos_reference_list_class_ref));
 
@@ -266,7 +283,7 @@ static void read_ref_list_into_cache(Block_Cache* cache)
 
 	memcpy(next_ptr->ivec, ivec, sizeof(next_ptr->ivec));  /* save this ivec in case this block has to be written back to disk */
 
-	nwos_read_object_from_disk_and_decrypt(&prev_ptr->next_block_ref, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]);
+	assert(nwos_read_object_from_disk_and_decrypt(&prev_ptr->next_block_ref, &next_ptr->dirty, FILE_BLOCK_SIZE, ivec, nwos_random_sequence[seq]));
 
 	for (i = 0; i < MAX_REFS_IN_SIDECAR; i++)
 	{
@@ -658,7 +675,7 @@ void nwos_add_to_references(ObjRef* ref, ObjRef* obj)
     /* first verify we are didn't pass in a reference list as the object */
     /* I.E. meant to call add_to_reference_list instead. */
 
-    nwos_get_object_class(obj, &class_ref);
+    nwos_get_object_class_without_update(obj, &class_ref);
     assert(!is_same_object(&class_ref, &nwos_reference_list_class_ref));
 
     nwos_read_object_headers_from_disk(obj, &header);
