@@ -27,6 +27,23 @@
 --
 --
 -- $Log: verify_file.c,v $
+-- Revision 1.21  2008/07/11 14:03:56  jsedwards
+-- Fix Feature Request #1996867 "Move verify_file --shred option warning
+-- before doing it".  Instead of moving the warning I removed the warning
+-- completely and changed the "--shred" option to "--remove-min-shred".
+-- I also changed the "--remove" option to "--remove-no-shred" so that it
+-- forces one to choose if they want shredding or not.  That way one is
+-- reminded of the choice before, instead of using --remove and then
+-- remembering it should have been shredded when it is too late.
+--
+-- Revision 1.20  2008/07/11 13:24:13  jsedwards
+-- Fix Bug #2004217 "verify -r fails if file is not a directory" - Added test
+-- to determine if the argument is a file or a directory and call the
+-- appropriate function.
+--
+-- Revision 1.19  2008/07/11 13:01:12  jsedwards
+-- Added include of limits.h to define PATH_MAX.
+--
 -- Revision 1.18  2008/06/23 04:46:13  jsedwards
 -- Added calls to nwos_normalize_path to clean up the paths for processing
 -- directories.
@@ -97,6 +114,7 @@
 
 #include <assert.h>
 #include <dirent.h>
+#include <limits.h>    /* define PATH_MAX */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -117,11 +135,18 @@ static void display_usage(char* prog_name)
     fprintf(stderr, "  --ignore-time\t\tdon't require time stamp to match.\n");
     fprintf(stderr, "  --public\t\tverify files against public data.\n");
     fprintf(stderr, "  -r, --recursive\trecurse through directories.\n");
-    fprintf(stderr, "  --remove\t\terase files if they verify correctly.\n");
-    fprintf(stderr, "  --shred\t\twrite random data over file and then remove.  See note below!\n");
+    fprintf(stderr, "  --remove-no-shred\terase files if they verify correctly.\n");
+    fprintf(stderr, "  --remove-min-shred\twrite random data over file and then remove.\n");
+    fprintf(stderr, "  --remove-max-shred\tNot implemented yet!\n");
 
-    fprintf(stderr, "PLEASE NOTE: the --shred option is very minimal in this version.  With the\n");
-    fprintf(stderr, "   proper equipment it is very possible that your data could still be read.\n");
+    fprintf(stderr, "\n");
+    fprintf(stderr, "*******************************************************************************\n");
+    fprintf(stderr, "  PLEASE NOTE: the --remove-min-shred option performs very minor shredding.\n");
+    fprintf(stderr, "  With the proper equipment it is very possible that your data could still be\n");
+    fprintf(stderr, "  read by someone.  You may want to consider using the GNU shred program until\n");
+    fprintf(stderr, "  I can incorporate the GNU shredding code into this program.\n");
+    fprintf(stderr, "*******************************************************************************\n");
+    fprintf(stderr, "\n");
 
     fprintf(stderr, "\n");
 }
@@ -396,6 +421,7 @@ int main(int argc, char* argv[])
     MatchCode match = MatchAll;
     char* path = DEFAULT_FILE;
     char normalized_path[PATH_MAX];
+    struct stat stat_struct;
 
     printf("\n");
 
@@ -407,26 +433,35 @@ int main(int argc, char* argv[])
 
     for (i = 1; i < argc && argv[i][0] == '-'; i++)
     {
-	if (strcmp(argv[i], "--remove") == 0)
+	if (strcmp(argv[i], "--remove-no-shred") == 0)
 	{
 	    if (remove_option == Shred_Before_Remove)
 	    {
-		fprintf(stderr, "NOTE: --shred option implies --remove\n");
+		fprintf(stderr, "Error: both --remove-no-shred and --remove-min-shred cannot be specified.\n\n");
+		exit(1);
 	    }
 	    else
 	    {
 		remove_option = Just_Remove;	// Don't shred it
 	    }
 	}
-	else if (strcmp(argv[i], "--shred") == 0)
+	else if (strcmp(argv[i], "--remove-min-shred") == 0)
 	{
 	    if (remove_option == Just_Remove)
 	    {
-		fprintf(stderr, "NOTE: --shred option implies --remove\n");
+		fprintf(stderr, "Error: both --remove-min-shred and --remove-no-shred cannot be specified.\n\n");
+		exit(1);
 	    }
 
 	    remove_option = Shred_Before_Remove;	// Do shred it
 	}
+	else if (strcmp(argv[i], "--remove-max-shred") == 0)
+	{
+	    fprintf(stderr, "Error: the --remove-max-shred option has not been implemented yet.  Use the\n");
+	    fprintf(stderr, "       --remove-min-shred for a very light shredding or use the GNU shred\n");
+	    fprintf(stderr, "       program instead, for a more thorough shredding.\n\n");
+	    exit(1);
+	}
 	else if (strcmp(argv[i], "--public") == 0)
 	{
 	    do_public = true;
@@ -517,7 +552,20 @@ int main(int argc, char* argv[])
 	    }
 	    else if (do_recursive)
 	    {
-		process_directory(normalized_path, match, remove_option);
+		if (lstat(normalized_path, &stat_struct) != 0)
+		{
+		    perror(normalized_path);
+		    exit(1);
+		}
+
+		if (S_ISDIR(stat_struct.st_mode))
+		{
+		    process_directory(normalized_path, match, remove_option);
+		}
+		else
+		{
+		    process_file(normalized_path, match, remove_option);
+		}
 	    }
 	    else
 	    {
@@ -526,16 +574,6 @@ int main(int argc, char* argv[])
 	}
     }
 
-    if (remove_option == Shred_Before_Remove)
-    {
-	printf("\n");
-	printf(" **************************************************************************************\n");
-	printf("  WARNING: the shredding done by this version of this program is very minimal!  It is\n");
-	printf("   very possible, with the right equipment, that someone could still read the files!\n");
-	printf(" *************************************************************************************\n");
-	printf("\n");
-    }
-
     nwos_terminate_objectify();
 
     return 0;
